/*
Name:       3DT_IFace_Nano.ino
Created:	24.02.2020 18:59:28
Author:     deHarry

Das Interface wird eingangsseitig (vom 3DTouch Sensor) parallel zum GRBL Probe-Eingang geschaltet und bekommt so 
die vom 3DTouch erzeugten Touch-Events mit.
Mit Erkennen eines Touch-Events wird ein Pin-Up Befehl ausgelst und so der Pin zurck gezogen. 
Gleichzeitig startet ein Timer und sendet nach Ablauf der eingestellten Zeit den Pin-Down Befehl 
an den 3DTouch, so dass der Sensor erneut ein Touch-Event auslsen kann.
Auf diese Weise kann der 3DTOuch/BLTouch mit seiner originalen Firmware auch in Verbindung mit GRBL an einer 
CNC Frse verwendet werden, um z.B. die Oberflche eines nichtleitenden Werkstcks abzutasten.

V1.0 - 24.02.2020
Diese erste Version arbeitet ohne Interrupts, rein pollend und ber delay() gesteuert. Bad practice aber luft ;-)

V1.1 - 22.10.2020
- Umstellung auf Interrrupt fr das Sensorsignal
- Erweiterung auf Einsatz auf einem Arduino (Nano oder Uno) - vorwiegend fr Debugzwecke und Programmentwicklung

V1.2 - 29.10.2020
- Der Touch-Impuls des 3DTouch ist zu kurz fr GRBL V1.1.
  Der AtTiny erkennt bereits, wann der 3DTouch ein Touch-Ebvent generiert, es muss nur noch ein zustzlicher 
  Ausgang des AtTiny (ich verwende PB1) parallel zum FET auf dem 3DTouch Interface-Board verdrahtet werden,
  der im inaktiven Zustand (kein Touch-Event) Hi-Z ist (also Input-Pullup) und bei Erkennen eines Touch-Events 
  den Pin fr eine einstellbare Zeit aktiv LOW schaltet.

  Man darf sich fragen, wie zeitkritisch die Erkennung des Probe-Signals in GRBL berhaupt ist.
  Wenn ein 10 ms langer Impuls nur hchst sporadisch erkannt wird, ist die Unsicherheit also mindestens 10 ms.
  GRBL kann auf dem AtMega328p bei 16 MHz Steps mit 15 kHz erzeugen.
  
V1.3 - 05.11.2020
- Touch-Impuls verlngert auf 900 ms
- Lnge berechnet, da die ms beim Digispark nicht korrekt erzeugt werden

  Die Routinen zur Bedienung der verschiedenen Pins knnen durch Register- und Bit-Operationen verkrzt
  werden. Das ist aber nicht notwendig, da das Timing auch mit den Standard Arduino-Implementierungen passt.
*/

//#define NANO		// Code fr Arduino Nano compilieren? -> Kommentarzeichen am Anfang der Zeile entfernen

// berprfung, ob der richtige Prozessortyp eingestellt ist
#if defined (NANO)
	#include "Servo.h"
	#if defined (__AVR_ATtiny85__) 
		#error "Achtung! Falscher Microcontroller ausgewaehlt..."
	#endif
#else
	#include "Servo8Bit.h"
	#if defined (__AVR_ATmega328P__) 
		#error "Achtung! Falscher Microcontroller ausgewaehlt..."
	#endif
#endif

// Definition von digitalPinToInterrupt() fr Digispark. Fr den AtMega (Arduino) ist das Makro bereits (anders) definiert
#ifndef digitalPinToInterrupt
	#define NOT_AN_INTERRUPT -1
	#define digitalPinToInterrupt(p) \
			((p) == 2 ? 0 : \
			((p) == 0 || (p) == 1 || (p) == 3 || (p) == 4 || (p) == 5) ? 1 : NOT_AN_INTERRUPT)
#endif

// Definition der Pins
#if defined (NANO)
	#define TOUCHSENSOR 2										// Eingang fuer Zmin-Signal vom Sensor
	#define SERVOOUT 8											// Steuer-Ausgang Richtung Sensor
	#define LED 13												// LED am Arduino Nano auf D13
	#define PULSEOUT 3											// Ausgang fr Puls-Verlngerung
#else
	#define TOUCHSENSOR 2										// Eingang fuer Zmin-Signal vom Sensor
	#define SERVOOUT 0											// Steuer-Ausgang Richtung Sensor
	#define LED 1												// LED am Digispark auf PB1
	#define PULSEOUT 3											// Ausgang fr Puls-Verlngerung
#endif

// Definitionen des Servosignals, Sxx = Winkel, Zahl = Mikrosekunden
#define S10 650													// Servo 10	- Push-pin DOWN (deploy probe)
#define S60 1165												// Servo 60	- Alarm Release & Touch SW Mode (M119)
#define S90 1475												// Servo 90	- Push-pin UP (retract probe)
#define S120 1780												// Servo 120	- Self test
#define S160 2190												// Servo 160	- Alarm Release & Push-pin UP

#define DOWN S10												// Zeiten des Servosignals in s fr entsprechende Winkel
#define UP S90

// Anmerkung: Die hier angegeben Zeiten stimmen beim Digispark nicht, sind etwa Faktor 4 lnger
#define WAITTIME 1100											// Zeit in ms, bis der Pin wieder ausgefahren wird (~800 ms)
#define PULSETIME 300											// Zeit in ms fr die Impulsverlngerung (~320 ms)

unsigned int iWaitTime;											// Variablen fr die beiden Zeiten
unsigned int iPulseTime;

// globale Variablen
unsigned long umillisecs = 0;									// millis() nur einmal ermitteln und merken
unsigned long utimePinDown = 0;									// Zeit nach Trigger bis Pin wieder ausgefahren wird
boolean bTimeRunning = false;									// Touchevent ist ausgeloest worden
volatile boolean bPinIsDown = false;							// Merker fuer "der Pin am Sensor ist ausgefahren"
volatile unsigned long eventHappened;							// Sensor meldet Touch Event
volatile boolean extendPulse = false;							// Sensor-Impuls muss verlngert werden

#ifdef NANO
	String inputString = "";               						 // Speicher fr Eingabestrings auf der seriellen SS
	boolean stringComplete = false;        						 // CR/LF empfangen

	#define LEDEIN (digitalWrite(LED, 1))
	#define LEDAUS (digitalWrite(LED, 0))
#else
	#define LEDEIN (digitalWrite(LED, 1))
	#define LEDAUS (digitalWrite(LED, 0))
#endif
#define LEDTOGGLE (digitalWrite(LED, !digitalRead(LED)))

// Instanz von Servo
#ifdef NANO
Servo servo;													// Servo Lib fr Arduino
#else
Servo8Bit servo;												// Servo Lib fr Tiny
#endif

// IRQ Routine
void SensorInput()
{
	eventHappened = millis();									// Zeit merken
	LEDEIN;
	servo.writeMicroseconds(UP);								// Pin einziehen...
	bPinIsDown = false;											// ...und merken
	
	pinMode(PULSEOUT, OUTPUT);									// Ausgang aktivieren
	digitalWrite(PULSEOUT, 0);									// Puls aktivieren (LOW aktiv)
	extendPulse = true;											// Impulsverlngerung vorbereiten, 2020-10-29
	
#if defined (NANO)
	Serial.println(".");										// Debugausgabe beim Nano
#endif
}

void setup()
{
// die Angaben aus den Defines mssen ca. geviertelt werden, um annhernd die gewnschten Zeiten zu ergeben
#ifdef ARDUINO_AVR_DIGISPARK		// gilt nur fr Digispark
	iWaitTime = (float)WAITTIME / 4.01;
	iPulseTime = (float)PULSETIME / 4.01;
#endif	
	// IOs konfigurieren
	pinMode(TOUCHSENSOR, INPUT);
	pinMode(SERVOOUT, OUTPUT);
	pinMode(LED, OUTPUT);
	pinMode(PULSEOUT, INPUT_PULLUP);							// Puls-Ausgang ist inaktiv Hi-Z

#ifdef NANO	
	Serial.begin(115200);										// auf dem Nano kann die serielle SS verwendet werden
#endif

	attachInterrupt(digitalPinToInterrupt(TOUCHSENSOR), SensorInput, RISING);		// hier wird der IRQ INT0 definiert, nicht der Pin !!!

	// 3DTouch Kommunikation initialisieren
	servo.attach(SERVOOUT);										// Ausgang fuer Servo/3DTouch aktivieren

	LEDAUS;														// LED definiert AUS

#ifdef NANO
	// Ausgabe der mglichen Eingaben ber die seriell SS fr Tests auf dem Arduino
	Serial.println("1 - Servo  10 - Push - pin DOWN(deploy probe)");
	Serial.println("2 - Servo  60 - Alarm Release & Touch SW Mode(M119)");
	Serial.println("3 - Servo  90 - Push - pin UP(retract probe)");
	Serial.println("4 - Servo 120 - Self test");
	Serial.println("5 - Servo 160 - Alarm Release & Push - pin UP");
#endif
}

void loop()
{
#ifdef NANO
	unsigned int variable = 0;
#endif

	umillisecs = millis();										// millis merken
	
	if (bPinIsDown == false)
	{
		 if ((umillisecs - eventHappened) > (unsigned long)iWaitTime)		// nach WAITTIME den Pin wieder ausfahren
		 {
			servo.writeMicroseconds(DOWN);
			LEDAUS;												// LED definiert AUS
			bPinIsDown = true;									// nach Trigger sollte 3DTouch den Pin automatisch einziehen
		 }
		 else 
		{
			if ((umillisecs - eventHappened) > (unsigned long)iPulseTime)	// nach PULSETIME den Ausgang wieder deaktivieren
			{
				digitalWrite(PULSEOUT, 1);						// Signal inaktiv (HIGH) schalten
				pinMode(PULSEOUT, INPUT_PULLUP);				// Ausgang auf Eingang schalten
				// digitalWrite(PULSEOUT, 0);					// Puls deaktivieren (Hi-Z)
				extendPulse = false;							// Impulsverlngerung abschalten
			}
		}
	}

#ifdef NANO
	// Sensor Kommandos manuell aufrufen
	if (stringComplete == true) 
	{
		if (inputString.length() > 0)
		{
			servo.attach(SERVOOUT);
			char value = (char)inputString.toInt();
			Serial.println(toAscii(value));
			switch (value)
			{
				case 1:
					variable = S10;
					Serial.println("Servo 10	- Push-pin DOWN (deploy probe)");
					break;
				case 2:
					variable = S60;
					Serial.println("Servo 60	- Alarm Release & Touch SW Mode (M119)");
					break;
				case 3:
					variable = S90;
					Serial.println("Servo 90	- Push-pin UP (retract probe)");
					break;
				case 4:
					variable = S120;
					Serial.println("Servo 120	- Self test");
					break;
				case 5:
					variable = S160;
					Serial.println("Servo 160	- Alarm Release & Push-pin UP");
					break;
				default:
					break;
			}
		}
		Serial.println((variable));
		servo.writeMicroseconds(variable);								// Sensor bedienen...

		inputString = "";
		stringComplete = false;
	}
#endif
}

// ------------------------------------------------------------------------------------------------
 // Abfrage der seriellen Schnittstelle
#ifdef NANO
void serialEvent() {
	while (Serial.available()) {
		// get the new byte:
		char inChar = (char)Serial.read();
		// add it to the inputString:
		inputString += inChar;
		// if the incoming character is a newline, set a flag
		// so the main loop can do something about it:
		if (inChar == '\r') {
			stringComplete = true;
		}
	}
}
#endif
